"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const hooks_1 = require("./hooks");
const hook_1 = require("./hook");
const utils_1 = require("./utils");
const factory = ({ type, className: baseClass, style, extend = (props) => (props), render = ({ children }) => (children) }) => {
    return react_1.forwardRef((props, ref) => {
        let { className, use } = props;
        const keys = new Set(['use']);
        const monitored = utils_1.monitor(props, keys);
        const classes = (style && style(monitored)) || null;
        const extended = (extend && extend(monitored)) || null;
        const children = (render && render(monitored)) || null;
        if (!type) {
            return children;
        }
        className = hooks_1.useClassName(baseClass, classes, className);
        const sanitized = utils_1.sanitize(keys, Object.assign(Object.assign({}, props), extended));
        props = Object.assign(Object.assign({}, sanitized), { children,
            className,
            ref });
        const intercepted = hook_1.interceptHooks(use, { type, props });
        return react_1.createElement(intercepted.type, intercepted.props);
    });
};
exports.default = factory;
