"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
exports.Type = {
    TYPE: 'type',
    PROPS: 'props'
};
const prepareHooks = (fns) => {
    const map = {};
    for (const { type, hook } of fns) {
        if (!map[type]) {
            map[type] = [];
        }
        map[type].push((args) => hook(args));
    }
    return map;
};
exports.interceptHooks = (use, context) => {
    if (!use) {
        return context;
    }
    const prepared = prepareHooks(use);
    for (const prop of Object.values(exports.Type)) {
        const fns = prepared[prop];
        if (!fns) {
            continue;
        }
        for (const fn of fns) {
            utils_1.define(context, prop, {
                value: fn(context[prop]),
                enumerable: true,
            });
        }
    }
    return context;
};
exports.createHook = (type) => (hook) => ({ type, hook });
